@@ -1,9 +1,13 @@
 /*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
+ * This program is copyright Alec Muffett 1993. The author disclaims all
+ * responsibility or liability with respect to it's usage or its effect
+ * upon hardware or computer systems, and maintains copyright as set out
+ * in the "LICENCE" document which accompanies distributions of Crack v4.0
+ * and upwards.
  */
 
+static char vers_id[] = "fascist.c : v2.3p3 Alec Muffett 14 dec 1997";
+
 #include "config.h"
 #include <sys/types.h>
 #include <errno.h>
@@ -32,6 +36,7 @@ typedef unsigned short uint16_t;
 
 #define MINDIFF 5
 #define MINLEN 6
+#define MAXSTEP 4
 
 #undef DEBUG
 #undef DEBUG2
@@ -43,7 +48,7 @@ static char *r_destructors[] = {
     ":",                        /* noop - must do this to test raw word. */
 
 #ifdef DEBUG2
-    NULL,
+    (char *) 0,
 #endif
 
     "[",                        /* trimming leading/trailing junk */
@@ -412,14 +417,14 @@ static char *r_destructors[] = {
     "/4s4h",
 
     /* done */
-    NULL
+    (char *) 0
 };
 
 static char *r_constructors[] = {
     ":",
 
 #ifdef DEBUG2
-    NULL,
+    (char *) 0,
 #endif
 
     "r",
@@ -428,7 +433,7 @@ static char *r_constructors[] = {
     "dr",
     "fr",
     "rf",
-    NULL
+    (char *) 0
 };
 
 int
@@ -492,40 +497,88 @@ GTry(rawtext, password)
 }
 
 char *
-FascistGecosUser(char *password, const char *user, const char *gecos)
+FascistGecos(password, uid)
+    char *password;
+    int uid;
 {
     int i;
     int j;
     int wc;
     char *ptr;
     int gwords;
+    struct passwd *pwp, passwd;
     char gbuffer[STRINGSIZE];
     char tbuffer[STRINGSIZE];
+    char *sbuffer = NULL;
+#ifdef HAVE_GETPWUID_R
+    size_t sbufferlen = LINE_MAX;
+#endif
     char *uwords[STRINGSIZE];
     char longbuffer[STRINGSIZE * 2];
 
-    if (gecos == NULL)
-	gecos = "";
+#ifdef HAVE_GETPWUID_R
+    sbuffer = malloc(sbufferlen);
+    if (sbuffer == NULL)
+    {
+        return ("memory allocation error");
+    }
+    while ((i = getpwuid_r(uid, &passwd, sbuffer, sbufferlen, &pwp)) != 0)
+    {
+        if (i == ERANGE)
+        {
+            free(sbuffer);
+
+	    sbufferlen += LINE_MAX;
+            sbuffer = malloc(sbufferlen);
+
+            if (sbuffer == NULL)
+            {
+                return ("memory allocation error");
+            }
+        } else {
+            pwp = NULL;
+            break;
+        }
+    }
+#else
+    /* Non-reentrant, but no choice since no _r routine */
+    pwp = getpwuid(uid);
+#endif
+
+    if (pwp == NULL)
+    {
+	if (sbuffer)
+	{
+		free(sbuffer);
+		sbuffer = NULL;
+	}
+	return _("you are not registered in the password file");
+    }
 
     /* lets get really paranoid and assume a dangerously long gecos entry */
 
-    strncpy(tbuffer, user, STRINGSIZE);
+    strncpy(tbuffer, pwp->pw_name, STRINGSIZE);
     tbuffer[STRINGSIZE-1] = '\0';
     if (GTry(tbuffer, password))
     {
+	if (sbuffer)
+	{
+		free(sbuffer);
+		sbuffer = NULL;
+	}
 	return _("it is based on your username");
     }
 
     /* it never used to be that you got passwd strings > 1024 chars, but now... */
 
-    strncpy(tbuffer, gecos, STRINGSIZE);
+    strncpy(tbuffer, pwp->pw_gecos, STRINGSIZE);
     tbuffer[STRINGSIZE-1] = '\0';
     strcpy(gbuffer, Lowercase(tbuffer));
 
     wc = 0;
     ptr = gbuffer;
     gwords = 0;
-    uwords[0] = NULL;
+    uwords[0] = (char *)0;
 
     while (*ptr)
     {
@@ -544,11 +597,11 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
 
 	if (wc == STRINGSIZE)
 	{
-	    uwords[--wc] = NULL;  /* to hell with it */
+	    uwords[--wc] = (char *) 0;  /* to hell with it */
 	    break;
 	} else
 	{
-	    uwords[wc] = NULL;
+	    uwords[wc] = (char *) 0;
 	}
 
 	while (*ptr && !ISSKIP(*ptr))
@@ -573,6 +626,11 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
     {
 	if (GTry(uwords[i], password))
 	{
+	    if (sbuffer)
+	    {
+	    	free(sbuffer);
+		sbuffer = NULL;
+	    }
 	    return _("it is based upon your password entry");
 	}
     }
@@ -588,6 +646,11 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
 
 	    if (GTry(longbuffer, password))
 	    {
+	        if (sbuffer)
+	        {
+	       	    free(sbuffer);
+		    sbuffer = NULL;
+	        }
 		return _("it is derived from your password entry");
 	    }
 
@@ -596,6 +659,11 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
 
 	    if (GTry(longbuffer, password))
 	    {
+	        if (sbuffer)
+	        {
+	       	    free(sbuffer);
+		    sbuffer = NULL;
+	        }
 		return _("it's derived from your password entry");
 	    }
 
@@ -605,6 +673,11 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
 
 	    if (GTry(longbuffer, password))
 	    {
+	        if (sbuffer)
+	        {
+	       	    free(sbuffer);
+		    sbuffer = NULL;
+	        }
 		return _("it is derivable from your password entry");
 	    }
 
@@ -614,82 +687,31 @@ FascistGecosUser(char *password, const char *user, const char *gecos)
 
 	    if (GTry(longbuffer, password))
 	    {
+	        if (sbuffer)
+	        {
+	       	    free(sbuffer);
+		    sbuffer = NULL;
+	        }
 		return _("it's derivable from your password entry");
 	    }
 	}
     }
 
-    return NULL;
-}
-
-char *
-FascistGecos(password, uid)
-    char *password;
-    int uid;
-{
-    struct passwd *pwp;
-    char *sbuffer = NULL;
-    char *ptr;
-
-#ifdef HAVE_GETPWUID_R
-    size_t sbufferlen = LINE_MAX;
-    struct passwd passwd;
-    int i;
-
-    sbuffer = malloc(sbufferlen);
-    if (sbuffer == NULL)
-    {
-	return ("memory allocation error");
-    }
-    while ((i = getpwuid_r(uid, &passwd, sbuffer, sbufferlen, &pwp)) != 0)
-    {
-	if (i == ERANGE)
-	{
-	    free(sbuffer);
-
-	    sbufferlen += LINE_MAX;
-	    sbuffer = malloc(sbufferlen);
-
-	    if (sbuffer == NULL)
-	    {
-		return ("memory allocation error");
-	    }
-	} else {
-	    pwp = NULL;
-	    break;
-	}
-    }
-#else
-    /* Non-reentrant, but no choice since no _r routine */
-    pwp = getpwuid(uid);
-#endif
-
-    if (pwp == NULL)
-    {
-	if (sbuffer)
-	{
-	    free(sbuffer);
-	    sbuffer = NULL;
-	}
-	return _("you are not registered in the password file");
-    }
-
-    ptr = FascistGecosUser(password, pwp->pw_name, pwp->pw_gecos);
-
     if (sbuffer)
     {
-	free(sbuffer);
-	sbuffer = NULL;
+        free(sbuffer);
+        sbuffer = NULL;
     }
 
-    return ptr;
+    return ((char *) 0);
 }
 
 char *
-FascistLookUser(PWDICT *pwp, char *instring,
-		const char *user, const char *gecos)
+FascistLook(pwp, instring)
+    PWDICT *pwp;
+    char *instring;
 {
-    int i,maxrepeat;
+    int i;
     char *ptr;
     char *jptr;
     char junk[STRINGSIZE];
@@ -756,10 +778,7 @@ FascistLookUser(PWDICT *pwp, char *instring,
 	ptr++;
     }
 
-    /*  Change by Ben Karsin from ITS at University of Hawaii at Manoa.  Static MAXSTEP
-        would generate many false positives for long passwords. */
-    maxrepeat = 3+(0.09*strlen(password));
-    if (i > maxrepeat)
+    if (i > MAXSTEP)
     {
 	return _("it is too simplistic/systematic");
     }
@@ -769,12 +788,7 @@ FascistLookUser(PWDICT *pwp, char *instring,
 	return _("it looks like a National Insurance number.");
     }
 
-    if (user != NULL)
-        ptr = FascistGecosUser(password, user, gecos);
-    else
-        ptr = FascistGecos(password, getuid());
-
-    if (ptr)
+    if (ptr = FascistGecos(password, getuid()))
     {
 	return (ptr);
     }
@@ -821,23 +835,13 @@ FascistLookUser(PWDICT *pwp, char *instring,
 	}
     }
 
-    return NULL;
-}
-
-char *
-FascistLook(pwp, instring)
-    PWDICT *pwp;
-    char *instring;
-{
-    return FascistLookUser(pwp, instring, NULL, NULL);
+    return ((char *) 0);
 }
 
 const char *
-FascistCheckUser(password, path, user, gecos)
+FascistCheck(password, path)
     const char *password;
     const char *path;
-    const char *user;
-    const char *gecos;
 {
     PWDICT *pwp;
     char pwtrunced[STRINGSIZE];
@@ -862,28 +866,17 @@ FascistCheckUser(password, path, user, gecos)
 
     if (!(pwp = PWOpen(path, "r")))
     {
-	return _("error loading dictionary");
+        /* shouldn't perror in a library or exit */
+	/* but should we return a "bad password" or "good password" if this error occurs */
+	perror("PWOpen");
+	exit(-1);
     }
 
     /* sure seems like we should close the database, since we're only likely to check one password */
-    res = FascistLookUser(pwp, pwtrunced, user, gecos);
+    res = FascistLook(pwp, pwtrunced);
 
     PWClose(pwp);
     pwp = (PWDICT *)0;
 
     return res;
 }
-
-const char *
-FascistCheck(password, path)
-    const char *password;
-    const char *path;
-{
-    return FascistCheckUser(password, path, NULL, NULL);
-}
-
-const char *
-GetDefaultCracklibDict()
-{
-    return DEFAULT_CRACKLIB_DICT;
-}
