@@ -1,24 +1,20 @@
 /*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
+ * This program is copyright Alec Muffett 1993. The author disclaims all 
+ * responsibility or liability with respect to it's usage or its effect 
+ * upon hardware or computer systems, and maintains copyright as set out 
+ * in the "LICENCE" document which accompanies distributions of Crack v4.0 
+ * and upwards.
  */
 
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#ifdef HAVE_ZLIB_H
-#include <zlib.h>
-#endif
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
 #include "packer.h"
 
-static const char vers_id[] = "packlib.c : v2.3p2 Alec Muffett 18 May 1993";
+static char vers_id[] = "packlib.c : v2.3p2 Alec Muffett 18 May 1993";
 
 #define DEBUG 0
 
@@ -34,15 +30,14 @@ struct pi_header64
 
 typedef struct
 {
-    void *ifp;
-    void *dfp;
-    void *wfp;
+    FILE *ifp;
+    FILE *dfp;
+    FILE *wfp;
     uint64_t flags;
     uint64_t hwms[256];
     struct pi_header64 header;
     int count;
-    char data_put[NUMWORDS][MAXWORDLEN];
-    char data_get[NUMWORDS][MAXWORDLEN];
+    char data[NUMWORDS][MAXWORDLEN];
 } PWDICT64;
 
 
@@ -72,14 +67,14 @@ PWOpen(prefix, mode)
     char iname[STRINGSIZE];
     char dname[STRINGSIZE];
     char wname[STRINGSIZE];
-    void *dfp;
-    void *ifp;
-    void *wfp;
+    FILE *dfp;
+    FILE *ifp;
+    FILE *wfp;
 
     if (pdesc.header.pih_magic == PIH_MAGIC)
     {
 	fprintf(stderr, "%s: another dictionary already open\n", prefix);
-	return NULL;
+	return ((PWDICT *) 0);
     }
 
     memset(&pdesc, '\0', sizeof(pdesc));
@@ -89,48 +84,17 @@ PWOpen(prefix, mode)
     snprintf(dname, STRINGSIZE, "%s.pwd", prefix);
     snprintf(wname, STRINGSIZE, "%s.hwm", prefix);
 
-    if (mode[0] == 'r')
+    if (!(pdesc.dfp = fopen(dname, mode)))
     {
-		pdesc.flags &= ~PFOR_USEZLIB;
-		/* first try the normal db file */
-		if (!(pdesc.dfp = fopen(dname, mode)))
-		{
-#ifdef HAVE_ZLIB_H
-			pdesc.flags |= PFOR_USEZLIB;
-			/* try extension .gz */
-			snprintf(dname, STRINGSIZE, "%s.pwd.gz", prefix);
-			if (!(pdesc.dfp = gzopen(dname, mode)))
-			{
-				perror(dname);
-				return NULL;
-			}
-#else
-		perror(dname);
-		return NULL;
-#endif
-		}
-	}
-	else
-	{
-		pdesc.flags &= ~PFOR_USEZLIB;
-		/* write mode: use fopen */
-		if (!(pdesc.dfp = fopen(dname, mode)))
-		{
-			perror(dname);
-			return NULL;
-		}
-	}
+	perror(dname);
+	return ((PWDICT *) 0);
+    }
 
     if (!(pdesc.ifp = fopen(iname, mode)))
     {
-#ifdef HAVE_ZLIB_H
-		if (pdesc.flags & PFOR_USEZLIB)
-			gzclose(pdesc.dfp);
-		else
-#endif
-			fclose(pdesc.dfp);
+	fclose(pdesc.dfp);
 	perror(iname);
-	return NULL;
+	return ((PWDICT *) 0);
     }
 
     if ((pdesc.wfp = fopen(wname, mode)))
@@ -160,17 +124,12 @@ PWOpen(prefix, mode)
 
 	    pdesc.header.pih_magic = 0;
 	    fclose(ifp);
-#ifdef HAVE_ZLIB_H
-		if (pdesc.flags & PFOR_USEZLIB)
-			gzclose(dfp);
-		else
-#endif
-			fclose(dfp);
-	    if (wfp)
+	    fclose(dfp);
+	    if(wfp)
 	    {
 		fclose(wfp);
 	    }
-	    return NULL;
+	    return ((PWDICT *) 0);
 	}
 
         if ((pdesc.header.pih_magic == 0) || (pdesc.header.pih_numwords == 0))
@@ -180,40 +139,29 @@ PWOpen(prefix, mode)
             if (!fread((char *) &pdesc64.header, sizeof(pdesc64.header), 1, ifp))
             {
                 fprintf(stderr, "%s: error reading header\n", prefix);
-
+ 
                 pdesc.header.pih_magic = 0;
                 fclose(ifp);
-#ifdef HAVE_ZLIB_H
-				if (pdesc.flags & PFOR_USEZLIB)
-					gzclose(dfp);
-				else
-#endif
-					fclose(dfp);
-		if (wfp)
+                fclose(dfp);
+		if(wfp)
 		{
 			fclose(wfp);
 		}
-                return NULL;
+                return ((PWDICT *) 0);
             }
             if (pdesc64.header.pih_magic != PIH_MAGIC)
             {
                 /* nope, not "64-bit" after all */
                 fprintf(stderr, "%s: error reading header\n", prefix);
-
+ 
                 pdesc.header.pih_magic = 0;
                 fclose(ifp);
-#ifdef HAVE_ZLIB_H
-				if (pdesc.flags & PFOR_USEZLIB)
-					gzclose(dfp);
-				else
-#endif
-					fclose(dfp);
-
-		if (wfp)
+                fclose(dfp);
+		if(wfp)
 		{
 			fclose(wfp);
 		}
-                return NULL;
+                return ((PWDICT *) 0);
             }
             pdesc.header.pih_magic = pdesc64.header.pih_magic;
             pdesc.header.pih_numwords = pdesc64.header.pih_numwords;
@@ -228,37 +176,26 @@ PWOpen(prefix, mode)
 
 	    pdesc.header.pih_magic = 0;
 	    fclose(ifp);
-#ifdef HAVE_ZLIB_H
-		if (pdesc.flags & PFOR_USEZLIB)
-			gzclose(dfp);
-		else
-#endif
-			fclose(dfp);
-
-	    if (wfp)
+	    fclose(dfp);
+	    if(wfp)
 	    {
 		fclose(wfp);
 	    }
-	    return NULL;
+	    return ((PWDICT *) 0);
 	}
 
         if (pdesc.header.pih_numwords < 1)
         {
             fprintf(stderr, "%s: invalid word count\n", prefix);
-
+ 
             pdesc.header.pih_magic = 0;
             fclose(ifp);
-#ifdef HAVE_ZLIB_H
-			if (pdesc.flags & PFOR_USEZLIB)
-				gzclose(dfp);
-			else
-#endif
-				fclose(dfp);
-	    if (wfp)
+            fclose(dfp);
+	    if(wfp)
 	    {
 		fclose(wfp);
 	    }
-            return NULL;
+            return ((PWDICT *) 0);
         }
 
 	if (pdesc.header.pih_blocklen != NUMWORDS)
@@ -267,17 +204,12 @@ PWOpen(prefix, mode)
 
 	    pdesc.header.pih_magic = 0;
 	    fclose(ifp);
-#ifdef HAVE_ZLIB_H
-		if (pdesc.flags & PFOR_USEZLIB)
-			gzclose(dfp);
-		else
-#endif
-			fclose(dfp);
-		if (wfp)
+	    fclose(dfp);
+	    if(wfp)
 	    {
 		fclose(wfp);
 	    }
-	    return NULL;
+	    return ((PWDICT *) 0);
 	}
 
 	if (pdesc.flags & PFOR_USEHWMS)
@@ -294,7 +226,7 @@ PWOpen(prefix, mode)
                 {
                     pdesc.hwms[i] = pdesc64.hwms[i];
                 }
-            }
+            } 
             else if (fread(pdesc.hwms, 1, sizeof(pdesc.hwms), wfp) != sizeof(pdesc.hwms))
 	    {
 		pdesc.flags &= ~PFOR_USEHWMS;
@@ -324,7 +256,7 @@ PWClose(pwp)
     if (pwp->flags & PFOR_WRITE)
     {
 	pwp->flags |= PFOR_FLUSH;
-	PutPW(pwp, NULL);	/* flush last index if necess */
+	PutPW(pwp, (char *) 0);	/* flush last index if necess */
 
 	if (fseek(pwp->ifp, 0L, 0))
 	{
@@ -356,13 +288,8 @@ PWClose(pwp)
     }
 
     fclose(pwp->ifp);
-#ifdef HAVE_ZLIB_H
-	if (pwp->flags & PFOR_USEZLIB)
-		gzclose(pwp->dfp);
-	else
-#endif
-		fclose(pwp->dfp);
-    if (pwp->wfp)
+    fclose(pwp->dfp);
+    if(pwp->wfp)
     {
         fclose(pwp->wfp);
     }
@@ -384,8 +311,8 @@ PutPW(pwp, string)
 
     if (string)
     {
-	strncpy(pwp->data_put[pwp->count], string, MAXWORDLEN);
-	pwp->data_put[pwp->count][MAXWORDLEN - 1] = '\0';
+	strncpy(pwp->data[pwp->count], string, MAXWORDLEN);
+	pwp->data[pwp->count][MAXWORDLEN - 1] = '\0';
 
 	pwp->hwms[string[0] & 0xff]= pwp->header.pih_numwords;
 
@@ -407,16 +334,16 @@ PutPW(pwp, string)
 
 	fwrite((char *) &datum, sizeof(datum), 1, pwp->ifp);
 
-	fputs(pwp->data_put[0], pwp->dfp);
+	fputs(pwp->data[0], pwp->dfp);
 	putc(0, pwp->dfp);
 
-	ostr = pwp->data_put[0];
+	ostr = pwp->data[0];
 
 	for (i = 1; i < NUMWORDS; i++)
 	{
 	    register int j;
 	    register char *nstr;
-	    nstr = pwp->data_put[i];
+	    nstr = pwp->data[i];
 
 	    if (nstr[0])
 	    {
@@ -429,7 +356,7 @@ PutPW(pwp, string)
 	    ostr = nstr;
 	}
 
-	memset(pwp->data_put, '\0', sizeof(pwp->data_put));
+	memset(pwp->data, '\0', sizeof(pwp->data));
 	pwp->count = 0;
     }
     return (0);
@@ -446,86 +373,72 @@ GetPW(pwp, number)
     register char *nstr;
     register char *bptr;
     char buffer[NUMWORDS * MAXWORDLEN];
+    static char data[NUMWORDS][MAXWORDLEN];
+    static uint32_t prevblock = 0xffffffff;
     uint32_t thisblock;
 
     thisblock = number / NUMWORDS;
 
+    if (prevblock == thisblock)
+    {
+#if DEBUG
+	fprintf(stderr, "returning (%s)\n", data[number % NUMWORDS]);
+#endif
+	return (data[number % NUMWORDS]);
+    }
+
     if (_PWIsBroken64(pwp->ifp))
     {
        uint64_t datum64;
        if (fseek(pwp->ifp, sizeof(struct pi_header64) + (thisblock * sizeof(uint64_t)), 0))
        {
            perror("(index fseek failed)");
-           return NULL;
+           return ((char *) 0);
        }
 
        if (!fread((char *) &datum64, sizeof(datum64), 1, pwp->ifp))
        {
            perror("(index fread failed)");
-           return NULL;
+           return ((char *) 0);
        }
        datum = datum64;
     } else {
        if (fseek(pwp->ifp, sizeof(struct pi_header) + (thisblock * sizeof(uint32_t)), 0))
        {
            perror("(index fseek failed)");
-           return NULL;
+           return ((char *) 0);
        }
 
        if (!fread((char *) &datum, sizeof(datum), 1, pwp->ifp))
        {
            perror("(index fread failed)");
-           return NULL;
+           return ((char *) 0);
        }
     }
 
-	int r = 1;
-#ifdef HAVE_ZLIB_H
-	if (pwp->flags & PFOR_USEZLIB)
-	{
-		r = gzseek(pwp->dfp, datum, 0);
-		if (r >= 0)
-			r = 0;
-	}
-	else
-#endif
-		r = fseek(pwp->dfp, datum, 0);
-
-
-    if (r)
+    if (fseek(pwp->dfp, datum, 0))
     {
 	perror("(data fseek failed)");
-	return NULL;
+	return ((char *) 0);
     }
-	r = 0;
 
-        memset(buffer, 0, sizeof(buffer));
-#ifdef HAVE_ZLIB_H
-	if (pwp->flags & PFOR_USEZLIB)
-	{
-		r = gzread(pwp->dfp, buffer, sizeof(buffer));
-		if (r < 0)
-			r = 0;
-	}
-	else
-#endif
-		r = fread(buffer, 1, sizeof(buffer), pwp->dfp);
-
-    if (!r)
+    if (!fread(buffer, 1, sizeof(buffer), pwp->dfp))
     {
 	perror("(data fread failed)");
-	return NULL;
+	return ((char *) 0);
     }
 
+    prevblock = thisblock;
+
     bptr = buffer;
 
-    for (ostr = pwp->data_get[0]; (*(ostr++) = *(bptr++)); /* nothing */ );
+    for (ostr = data[0]; (*(ostr++) = *(bptr++)); /* nothing */ );
 
-    ostr = pwp->data_get[0];
+    ostr = data[0];
 
     for (i = 1; i < NUMWORDS; i++)
     {
-	nstr = pwp->data_get[i];
+	nstr = data[i];
 	strcpy(nstr, ostr);
 
 	ostr = nstr + *(bptr++);
@@ -534,10 +447,10 @@ GetPW(pwp, number)
 	ostr = nstr;
     }
 
-    return (pwp->data_get[number % NUMWORDS]);
+    return (data[number % NUMWORDS]);
 }
 
-unsigned int
+uint32_t
 FindPW(pwp, string)
     PWDICT *pwp;
     char *string;
@@ -579,12 +492,11 @@ fprintf(stderr, "look for (%s)\n", string);
     fprintf(stderr, "---- %lu, %lu ----\n", lwm, hwm);
 #endif
 
-    middle = lwm + ((hwm - lwm + 1) / 2);
-
     for (;;)
     {
 	int cmp;
 
+	middle = lwm + ((hwm - lwm + 1) / 2);
 
 #if DEBUG
 	fprintf(stderr, "lwm = %lu,  middle = %lu,  hwm = %lu\n", lwm, middle, hwm);
@@ -613,7 +525,7 @@ fprintf(stderr, "look for (%s)\n", string);
 
         if (middle == hwm)
         {
-#if DEBUG
+#if DEBUG 
 		fprintf(stderr, "at terminal subdivision, stopping search\n");
 #endif
 		break;
@@ -622,13 +534,11 @@ fprintf(stderr, "look for (%s)\n", string);
 	if (cmp < 0)
 	{
 	    hwm = middle;
-	    middle = lwm + ((hwm - lwm ) / 2);
-	}
+	} 
 	else if (cmp > 0)
 	{
 	    lwm = middle;
-	    middle = lwm + ((hwm - lwm + 1) / 2);
-	}
+	} 
     }
 
     return (PW_WORDS(pwp));
