@@ -1,10 +1,12 @@
 /*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
+ * This program is copyright Alec Muffett 1993. The author disclaims all 
+ * responsibility or liability with respect to it's usage or its effect 
+ * upon hardware or computer systems, and maintains copyright as set out 
+ * in the "LICENCE" document which accompanies distributions of Crack v4.0 
+ * and upwards.
  */
 
-static const char vers_id[] = "rules.c : v5.0p3 Alec Muffett 20 May 1993";
+static char vers_id[] = "rules.c : v5.0p3 Alec Muffett 20 May 1993";
 
 #include "config.h"
 #include <string.h>
@@ -24,7 +26,7 @@ Debug(val, a, b, c, d, e, f, g)
     int val;
     char *a, *b, *c, *d, *e, *f, *g;
 {
-    fprintf(stderr, a, b, c, d, e, f, g);
+    fprintf(stderr, a, b, c, d, e, f);
 }
 #else
 static void
@@ -368,7 +370,7 @@ PolyStrchr(string, class)
 	}
 	string++;
     }
-    return NULL;
+    return ((char *) 0);
 }
 
 char *
@@ -471,18 +473,18 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1])
 	    {
 		Debug(1, "Mangle: '>' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		limit = Char2Int(*(++ptr));
 		if (limit < 0)
 		{
 		    Debug(1, "Mangle: '>' weird argument in '%s'\n", control);
-		    return NULL;
+		    return ((char *) 0);
 		}
-		if ( (int) strlen(area) <= limit)
+		if (strlen(area) <= limit)
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 	    }
 	    break;
@@ -490,18 +492,18 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1])
 	    {
 		Debug(1, "Mangle: '<' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		limit = Char2Int(*(++ptr));
 		if (limit < 0)
 		{
 		    Debug(1, "Mangle: '<' weird argument in '%s'\n", control);
-		    return NULL;
+		    return ((char *) 0);
 		}
-		if ( (int) strlen(area) >= limit)
+		if (strlen(area) >= limit)
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 	    }
 	    break;
@@ -509,7 +511,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1])
 	    {
 		Debug(1, "Mangle: prepend missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		area2[0] = *(++ptr);
@@ -521,7 +523,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1])
 	    {
 		Debug(1, "Mangle: append missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register char *string;
@@ -535,7 +537,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || !ptr[2])
 	    {
 		Debug(1, "Mangle: extract missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register int i;
@@ -546,7 +548,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		if (start < 0 || length < 0)
 		{
 		    Debug(1, "Mangle: extract: weird argument in '%s'\n", control);
-		    return NULL;
+		    return ((char *) 0);
 		}
 		strcpy(area2, area);
 		for (i = 0; length-- && area2[start + i]; i++)
@@ -561,7 +563,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || !ptr[2])
 	    {
 		Debug(1, "Mangle: overstrike missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register int i;
@@ -570,7 +572,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		{
 		    Debug(1, "Mangle: overstrike weird argument in '%s'\n",
 			  control);
-		    return NULL;
+		    return ((char *) 0);
 		} else
 		{
 		    ++ptr;
@@ -585,7 +587,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || !ptr[2])
 	    {
 		Debug(1, "Mangle: insert missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register int i;
@@ -596,7 +598,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		{
 		    Debug(1, "Mangle: insert weird argument in '%s'\n",
 			  control);
-		    return NULL;
+		    return ((char *) 0);
 		}
 		p1 = area;
 		p2 = area2;
@@ -616,7 +618,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || (ptr[1] == RULE_CLASS && !ptr[2]))
 	    {
 		Debug(1, "Mangle: delete missing arguments in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else if (ptr[1] != RULE_CLASS)
 	    {
 		strcpy(area, Purge(area, *(++ptr)));
@@ -630,7 +632,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || !ptr[2] || (ptr[1] == RULE_CLASS && !ptr[3]))
 	    {
 		Debug(1, "Mangle: subst missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else if (ptr[1] != RULE_CLASS)
 	    {
 		strcpy(area, Substitute(area, ptr[1], ptr[2]));
@@ -645,18 +647,18 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || (ptr[1] == RULE_CLASS && !ptr[2]))
 	    {
 		Debug(1, "Mangle: '/' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else if (ptr[1] != RULE_CLASS)
 	    {
 		if (!strchr(area, *(++ptr)))
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 	    } else
 	    {
 		if (!PolyStrchr(area, ptr[2]))
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 		ptr += 2;
 	    }
@@ -665,18 +667,18 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || (ptr[1] == RULE_CLASS && !ptr[2]))
 	    {
 		Debug(1, "Mangle: '!' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else if (ptr[1] != RULE_CLASS)
 	    {
 		if (strchr(area, *(++ptr)))
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 	    } else
 	    {
 		if (PolyStrchr(area, ptr[2]))
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 		ptr += 2;
 	    }
@@ -694,28 +696,28 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || !ptr[2] || (ptr[2] == RULE_CLASS && !ptr[3]))
 	    {
 		Debug(1, "Mangle: '=' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register int i;
 		if ((i = Char2Int(ptr[1])) < 0)
 		{
 		    Debug(1, "Mangle: '=' weird argument in '%s'\n", control);
-		    return NULL;
+		    return ((char *) 0);
 		}
 		if (ptr[2] != RULE_CLASS)
 		{
 		    ptr += 2;
 		    if (area[i] != *ptr)
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		} else
 		{
 		    ptr += 3;
 		    if (!MatchClass(*ptr, area[i]))
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		}
 	    }
@@ -746,7 +748,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || (ptr[1] == RULE_CLASS && !ptr[2]))
 	    {
 		Debug(1, "Mangle: '(' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		if (ptr[1] != RULE_CLASS)
@@ -754,14 +756,14 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		    ptr++;
 		    if (area[0] != *ptr)
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		} else
 		{
 		    ptr += 2;
 		    if (!MatchClass(*ptr, area[0]))
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		}
 	    }
@@ -769,7 +771,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 	    if (!ptr[1] || (ptr[1] == RULE_CLASS && !ptr[2]))
 	    {
 		Debug(1, "Mangle: ')' missing argument in '%s'\n", control);
-		return NULL;
+		return ((char *) 0);
 	    } else
 	    {
 		register int i;
@@ -781,7 +783,7 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		    i--;
 		} else
 		{
-		    return NULL;
+		    return ((char *) 0);
 		}
 
 		if (ptr[1] != RULE_CLASS)
@@ -789,27 +791,27 @@ Mangle(input, control)		/* returns a pointer to a controlled Mangle */
 		    ptr++;
 		    if (area[i] != *ptr)
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		} else
 		{
 		    ptr += 2;
 		    if (!MatchClass(*ptr, area[i]))
 		    {
-			return NULL;
+			return ((char *) 0);
 		    }
 		}
 	    }
 
 	default:
 	    Debug(1, "Mangle: unknown command %c in %s\n", *ptr, control);
-	    return NULL;
+	    return ((char *) 0);
 	    break;
 	}
     }
     if (!area[0])		/* have we deweted de poor widdle fing away? */
     {
-	return NULL;
+	return ((char *) 0);
     }
     return (area);
 }
