@@ -1,7 +1,9 @@
 /*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
+ * This program is copyright Alec Muffett 1993. The author disclaims all 
+ * responsibility or liability with respect to it's usage or its effect 
+ * upon hardware or computer systems, and maintains copyright as set out 
+ * in the "LICENCE" document which accompanies distributions of Crack v4.0 
+ * and upwards.
  */
 
 #include "config.h"
@@ -11,7 +13,7 @@
 #endif
 #include "packer.h"
 
-static const char vers_id[] = "stringlib.c : v2.3p2 Alec Muffett 18 May 1993";
+static char vers_id[] = "stringlib.c : v2.3p2 Alec Muffett 18 May 1993";
 
 char
 Chop(string)
