@@ -1,13 +1,6 @@
-/*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
- */
-
 #include <stdio.h>
 #include <limits.h>
 #include <string.h>
-#include <locale.h>
 
 #define IN_CRACKLIB
 #include "config.h"
@@ -21,9 +14,6 @@ main(int argc, char **argv)
 	const char *why;
 	int i;
 
-	setlocale(LC_ALL, "");
-	textdomain(PACKAGE);
-
 	while (fgets(buf, sizeof(buf), stdin) != NULL) {
 		while (((i = strlen(buf)) > 0) && (i > 0)) {
 			if (strchr("\r\n", buf[i - 1]) != NULL) {
