@@ -1,7 +1,9 @@
 /*
- * This program is copyright Alec Muffett 1993, portions copyright other authors.
- * The authors disclaim all responsibility or liability with respect to it's usage
- * or its effect upon hardware or computer systems.
+ * This program is copyright Alec Muffett 1993. The author disclaims all 
+ * responsibility or liability with respect to it's usage or its effect 
+ * upon hardware or computer systems, and maintains copyright as set out 
+ * in the "LICENCE" document which accompanies distributions of Crack v4.0 
+ * and upwards.
  */
 
 #include <stdio.h>
@@ -17,8 +19,8 @@ main(argc, argv)
     int argc;
     char *argv[];
 {
-    unsigned long readed;
-    unsigned long wrote;
+    uint32_t readed;
+    uint32_t wrote;
     PWDICT *pwp;
     char buffer[STRINGSIZE];
     char *file;
@@ -27,7 +29,7 @@ main(argc, argv)
     {
 	file = DEFAULT_CRACKLIB_DICT;
     }
-    else
+    else 
     {
         file = argv[1];
     }
@@ -57,13 +59,13 @@ main(argc, argv)
 
 	if (!buffer[0])
 	{
-	    fprintf(stderr, "skipping line: %lu\n", readed);
+	    fprintf(stderr, "skipping line: %u\n", readed);
 	    continue;
 	}
 
 	if (PutPW(pwp, buffer))
 	{
-	    fprintf(stderr, "error: PutPW '%s' line %luy\n", buffer, readed);
+	    fprintf(stderr, "error: PutPW '%s' line %uy\n", buffer, readed);
 	}
 
 	wrote++;
@@ -71,7 +73,7 @@ main(argc, argv)
 
     PWClose(pwp);
 
-    printf("%lu %lu\n", readed, wrote);
+    printf("%u %u\n", readed, wrote);
 
     return (0);
 }
